@@ -25,16 +25,17 @@
 
 #ifndef _DEF_ACC_DEVICE_T
 #define _DEF_ACC_DEVICE_T
+#include "nvhpc_acc_device_type.h"
 typedef enum {
-  acc_device_none = 0,
-  acc_device_default = 1,
-  acc_device_host = 2,
-  acc_device_not_host = 3,
-  acc_device_nvidia = 4,
-  acc_device_pgi_opencl = 7,
-  acc_device_nvidia_opencl = 8,
-  acc_device_opencl = 9,
-  acc_device_current = 10
+  acc_device_none = nvhpc_acc_device_none,
+  acc_device_default = nvhpc_acc_device_default,
+  acc_device_host = nvhpc_acc_device_host,
+  acc_device_not_host = nvhpc_acc_device_not_host,
+  acc_device_nvidia = nvhpc_acc_device_nvidia,
+  acc_device_pgi_opencl = nvhpc_acc_device_RESERVED7,
+  acc_device_nvidia_opencl = nvhpc_acc_device_RESERVED8,
+  acc_device_opencl = nvhpc_acc_device_RESERVED9,
+  acc_device_current = nvhpc_acc_device_current
 } acc_device_t;
 #endif
 #ifndef _DEF_ACC_DEVICE_PROPERTY_T
@@ -48,17 +49,6 @@ typedef enum {
   acc_property_driver = 5
 } acc_device_property_t;
 #endif
-#ifndef _DEF_ACC_MANAGED_MEM_ADVISE_T
-#define _DEF_ACC_MANAGED_MEM_ADVISE_T
-typedef enum {
-  acc_managed_set_read_mostly = 0,
-  acc_managed_unset_read_mostly = 1,
-  acc_managed_set_preferred_location = 2,
-  acc_managed_unset_preferred_location = 3,
-  acc_managed_set_accessed_by = 4,
-  acc_managed_unset_accessed_by = 5
-} acc_managed_mem_advice_t;
-#endif
 #ifndef _DEF_ACCX_MEM_MODE_T
 #define _DEF_ACCX_MEM_MODE_T
 typedef enum {
@@ -76,7 +66,42 @@ typedef enum {
   accx_mem_hints_enable_all       /* memory hints enabled at explicit and implicit data constructs */
 } accx_mem_hints_t;
 #endif
-
+#ifndef _DEF_ACCX_MEM_PREFETCH_T
+#define _DEF_ACCX_MEM_PREFETCH_T
+typedef enum {
+  accx_mem_prefetch_default = 0,     /* runtime decides, can be set with environment variable */
+  accx_mem_prefetch_disable,         /* memory prefetch disabled */
+  accx_mem_prefetch_enable_update,   /* memory prefetch enabled at update data constructs only */
+  accx_mem_prefetch_enable_explicit, /* memory prefetch enabled at explicit data constructs only */
+  accx_mem_prefetch_enable_all       /* memory prefetch enabled at explicit and implicit data constructs */
+} accx_mem_prefetch_t;
+#endif
+#ifndef _DEF_ACCX_MEM_ADVISE_T
+#define _DEF_ACCX_MEM_ADVISE_T
+typedef enum {
+  accx_mem_advise_set_read_mostly = 0,
+  accx_mem_advise_unset_read_mostly,
+  accx_mem_advise_set_preferred_location,
+  accx_mem_advise_unset_preferred_location,
+  accx_mem_advise_set_accessed_by,
+  accx_mem_advise_unset_accessed_by,
+} accx_mem_advise_t;
+#endif
+#ifndef _DEF_ACCX_MEM_ADVISE_LOCATION_T
+#define _DEF_ACCX_MEM_ADVISE_LOCATION_T
+typedef enum {
+  accx_mem_advise_location_none = 0,  /* location not set */
+  accx_mem_advise_location_host,      /* location host */
+  accx_mem_advise_location_device     /* location device */
+} accx_mem_advise_location_t;
+#endif
+#ifndef _DEF_ACCX_MEM_PREFETCH_LOCATION_T
+#define _DEF_ACCX_MEM_PREFETCH_LOCATION_T
+typedef enum {
+  accx_mem_prefetch_location_host = 0,   /* prefetch location is host */
+  accx_mem_prefetch_location_device      /* prefetch location is device */
+} accx_mem_prefetch_location_t;
+#endif
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -84,8 +109,8 @@ extern "C" {
 /* size_t */
 #include <stdio.h>
 
-void acc_set_default_async(int async);
-int acc_get_default_async(void);
+extern void acc_set_default_async(int async);
+extern int acc_get_default_async(void);
 extern void acc_set_host_only();
 extern int acc_get_num_devices(acc_device_t devtype);
 extern acc_device_t acc_get_device(void);
@@ -96,7 +121,9 @@ extern size_t acc_get_property(int devnum, acc_device_t devtype,
 extern const char *acc_get_property_string(int devnum, acc_device_t devtype,
                                            acc_device_property_t property);
 extern void acc_init(acc_device_t devtype);
+extern void acc_init_device(int devnum, acc_device_t devtype);
 extern void acc_shutdown(acc_device_t devtype);
+extern void acc_shutdown_device(int devnum, acc_device_t devtype);
 extern void acc_set_deviceid(int devid);
 extern int acc_get_deviceid(int devnum, acc_device_t devtype);
 extern int acc_async_test(__PGI_LLONG async);
@@ -282,8 +309,13 @@ extern void accx_set_debug(int debug);
 extern void accx_set_synchronous(int synchronous);
 extern void accx_set_mem_mode(accx_mem_mode_t value);
 extern void accx_set_mem_hints(accx_mem_hints_t value);
-extern int accx_managed_mem_advise(void *devptr, size_t size,
-                                   acc_managed_mem_advice_t advice);
+extern void accx_set_mem_prefetch(accx_mem_prefetch_t value);
+extern void accx_mem_advise(accx_mem_advise_t advice,
+                            accx_mem_advise_location_t location,
+                            void *ptr, __PGI_ULLONG bytes);
+extern void accx_mem_prefetch(accx_mem_prefetch_location_t location,
+                              void *ptr,  __PGI_ULLONG bytes,
+                              __PGI_LLONG async);
 typedef struct { void *graph; void *graph_exec; } acc_graph_t;
 extern void accx_async_begin_capture(__PGI_LLONG async);
 extern void accx_async_begin_capture_auto(__PGI_LLONG checksum, __PGI_LLONG async);
