@@ -20,5 +20,6 @@
 namespace exec {
   // A simple scheduler that executes its continuation inline, on the
   // thread of the caller of start().
-  using inline_scheduler = stdexec::__inln::__scheduler;
+  using inline_scheduler
+    [[deprecated("Use stdexec::inline_scheduler instead")]] = stdexec::inline_scheduler;
 } // namespace exec
