@@ -62,7 +62,7 @@ namespace exec {
 
     inline void __throw_error_code_if(bool __cond, int __ec) {
       if (__cond) {
-        throw std::system_error(__ec, std::system_category());
+        STDEXEC_THROW(std::system_error(__ec, std::system_category()));
       }
     }
 
@@ -116,7 +116,7 @@ namespace exec {
         }
       }
 
-      static ::io_uring_params __init_params(unsigned __flags) noexcept {
+      static auto __init_params(unsigned __flags) noexcept -> ::io_uring_params {
         ::io_uring_params __params{};
         __params.flags = __flags;
         return __params;
@@ -205,7 +205,7 @@ namespace exec {
       }
 
       // This function submits the given queue of tasks to the io_uring.
-      //
+
       // Each task that is ready to be completed is moved to the __ready queue.
       // If the submission queue gets full before all tasks are submitted, the
       // remaining tasks are moved to the __pending queue.
@@ -231,10 +231,9 @@ namespace exec {
           } else {
             __op->__vtable_->__submit_(__op, __sqe);
 #    ifdef STDEXEC_HAS_IO_URING_ASYNC_CANCELLATION
-            if (__is_stopped && __sqe.opcode != IORING_OP_ASYNC_CANCEL) {
-#    else
-            if (__is_stopped) {
+            __is_stopped = __is_stopped && __sqe.opcode != IORING_OP_ASYNC_CANCEL;
 #    endif
+            if (__is_stopped) {
               __stop(__op);
             } else {
               __sqe.user_data = bit_cast<__u64>(__op);
@@ -362,24 +361,34 @@ namespace exec {
         , __wakeup_operation_{this, __eventfd_} {
       }
 
-      void wakeup() {
+      auto try_wakeup() noexcept -> std::error_code {
         std::uint64_t __wakeup = 1;
-        __throw_error_code_if(::write(__eventfd_, &__wakeup, sizeof(__wakeup)) == -1, errno);
+        if (::write(__eventfd_, &__wakeup, sizeof(__wakeup)) == -1) {
+          return {errno, std::system_category()};
+        }
+        return {};
+      }
+
+      void wakeup() {
+        if (auto __ec = try_wakeup()) {
+          STDEXEC_THROW(std::system_error{__ec});
+        }
       }
 
       /// @brief Resets the io context to its initial state.
       void reset() {
         if (__is_running_.load(std::memory_order_relaxed) || __n_total_submitted_ > 0) {
-          throw std::runtime_error("exec::io_uring_context::reset() called on a running context");
+          STDEXEC_THROW(
+            std::runtime_error("exec::io_uring_context::reset() called on a running context"));
         }
         __n_submissions_in_flight_.store(0, std::memory_order_relaxed);
         __stop_source_.reset();
         __stop_source_.emplace();
       }
 
-      void request_stop() {
+      auto request_stop() noexcept -> std::error_code {
         __stop_source_->request_stop();
-        wakeup();
+        return try_wakeup();
       }
 
       auto stop_requested() const noexcept -> bool {
@@ -433,7 +442,7 @@ namespace exec {
         __n_total_submitted_ -= __completion_queue_.complete();
         STDEXEC_ASSERT(
           0 <= __n_total_submitted_
-          && __n_total_submitted_ <= static_cast<std::ptrdiff_t>(__params_.cq_entries));
+          && std::cmp_less_equal(__n_total_submitted_, __params_.cq_entries));
         __u32 __max_submissions = __params_.cq_entries - static_cast<__u32>(__n_total_submitted_);
         __pending_.append(__requests_.pop_all_reversed());
         __submission_result __result = __submission_queue_.submit(
@@ -442,11 +451,11 @@ namespace exec {
           __stop_source_->stop_requested());
         __n_total_submitted_ += __result.__n_submitted;
         __n_newly_submitted_ += __result.__n_submitted;
-        STDEXEC_ASSERT(__n_total_submitted_ <= static_cast<std::ptrdiff_t>(__params_.cq_entries));
+        STDEXEC_ASSERT(std::cmp_less_equal(__n_total_submitted_, __params_.cq_entries));
         __pending_ = static_cast<__task_queue&&>(__result.__pending);
         while (!__result.__ready.empty()) {
-          __n_total_submitted_ -=
-            __completion_queue_.complete(static_cast<__task_queue&&>(__result.__ready));
+          __n_total_submitted_ -= __completion_queue_
+                                    .complete(static_cast<__task_queue&&>(__result.__ready));
           STDEXEC_ASSERT(0 <= __n_total_submitted_);
           __pending_.append(__requests_.pop_all_reversed());
           __max_submissions = __params_.cq_entries - static_cast<__u32>(__n_total_submitted_);
@@ -456,7 +465,7 @@ namespace exec {
             __stop_source_->stop_requested());
           __n_total_submitted_ += __result.__n_submitted;
           __n_newly_submitted_ += __result.__n_submitted;
-          STDEXEC_ASSERT(__n_total_submitted_ <= static_cast<std::ptrdiff_t>(__params_.cq_entries));
+          STDEXEC_ASSERT(std::cmp_less_equal(__n_total_submitted_, __params_.cq_entries));
           __pending_ = static_cast<__task_queue&&>(__result.__pending);
         }
       }
@@ -464,9 +473,10 @@ namespace exec {
       void run_until_stopped() {
         bool expected_running = false;
         // Only one thread of execution is allowed to drive the io context.
-        if (!__is_running_.compare_exchange_strong(
-              expected_running, true, std::memory_order_relaxed)) {
-          throw std::runtime_error("exec::io_uring_context::run() called on a running context");
+        if (!__is_running_
+               .compare_exchange_strong(expected_running, true, std::memory_order_relaxed)) {
+          STDEXEC_THROW(
+            std::runtime_error("exec::io_uring_context::run() called on a running context"));
         } else {
           // Check whether we restart the context after a context-wide stop.
           // We have to reset the stop source in this case.
@@ -480,9 +490,8 @@ namespace exec {
             __wakeup_operation_.start();
           }
         }
-        scope_guard __not_running{[&]() noexcept {
-          __is_running_.store(false, std::memory_order_relaxed);
-        }};
+        scope_guard __not_running{
+          [&]() noexcept { __is_running_.store(false, std::memory_order_relaxed); }};
         __pending_.append(__requests_.pop_all_reversed());
         while (__n_total_submitted_ > 0 || !__pending_.empty()) {
           run_some();
@@ -495,7 +504,7 @@ namespace exec {
           constexpr int __min_complete = 1;
           STDEXEC_ASSERT(
             0 <= __n_total_submitted_
-            && __n_total_submitted_ <= static_cast<std::ptrdiff_t>(__params_.cq_entries));
+            && std::cmp_less_equal(__n_total_submitted_, __params_.cq_entries));
           int rc = __io_uring_enter(
             __ring_fd_,
             static_cast<unsigned>(__n_newly_submitted_),
@@ -539,7 +548,9 @@ namespace exec {
         __context& __context_;
 
         void operator()() const noexcept {
-          __context_.request_stop();
+          if (auto __ec = __context_.request_stop()) {
+            std::terminate();
+          }
         }
       };
 
@@ -551,21 +562,24 @@ namespace exec {
         __context& __context_;
         until __mode_;
 
-        using __on_stopped_callback = typename stdexec::stop_token_of_t<
-          stdexec::env_of_t<_Rcvr&>>::template callback_type<__on_stop>;
+        using __on_stopped_callback = stdexec::stop_callback_for_t<
+          stdexec::stop_token_of_t<stdexec::env_of_t<_Rcvr&>>,
+          __on_stop
+        >;
 
         void start() & noexcept {
           std::optional<__on_stopped_callback> __callback(
             std::in_place,
             stdexec::get_stop_token(stdexec::get_env(__rcvr_)),
             __on_stop{__context_});
-          try {
+          STDEXEC_TRY {
             if (__mode_ == until::stopped) {
               __context_.run_until_stopped();
             } else {
               __context_.run_until_empty();
             }
-          } catch (...) {
+          }
+          STDEXEC_CATCH_ALL {
             __callback.reset();
             stdexec::set_error(static_cast<_Rcvr&&>(__rcvr_), std::current_exception());
           }
@@ -584,7 +598,8 @@ namespace exec {
         using completion_signatures = stdexec::completion_signatures<
           stdexec::set_value_t(),
           stdexec::set_error_t(std::exception_ptr),
-          stdexec::set_stopped_t()>;
+          stdexec::set_stopped_t()
+        >;
 
         template <stdexec::receiver_of<completion_signatures> _Rcvr>
         auto connect(_Rcvr __rcvr) const noexcept -> __run_op<_Rcvr> {
@@ -640,23 +655,19 @@ namespace exec {
     }
 
     template <class _Op>
-    concept __io_task = //
-      requires(_Op& __op, ::io_uring_sqe& __sqe, const ::io_uring_cqe& __cqe) {
-        { __op.context() } noexcept -> std::convertible_to<__context&>;
-        { __op.ready() } noexcept -> std::convertible_to<bool>;
-        { __op.submit(__sqe) } noexcept;
-        { __op.complete(__cqe) } noexcept;
-      };
+    concept __io_task = requires(_Op& __op, ::io_uring_sqe& __sqe, const ::io_uring_cqe& __cqe) {
+      { __op.context() } noexcept -> std::convertible_to<__context&>;
+      { __op.ready() } noexcept -> std::convertible_to<bool>;
+      { __op.submit(__sqe) } noexcept;
+      { __op.complete(__cqe) } noexcept;
+    };
 
     template <class _Op>
-    concept __stoppable_task = //
-      __io_task<_Op> &&        //
-      requires(_Op& __op) {
-        {
-          static_cast<_Op&&>(__op).receiver()
-        } noexcept
-          -> stdexec::receiver_of<stdexec::completion_signatures<stdexec::set_stopped_t()>>;
-      };
+    concept __stoppable_task = __io_task<_Op> && requires(_Op& __op) {
+      {
+        static_cast<_Op &&>(__op).receiver()
+      } noexcept -> stdexec::receiver_of<stdexec::completion_signatures<stdexec::set_stopped_t()>>;
+    };
 
     template <__stoppable_task _Op>
     using __receiver_of_t = stdexec::__decay_t<decltype(std::declval<_Op&>().receiver())>;
@@ -682,7 +693,7 @@ namespace exec {
 
       template <class... _Args>
         requires stdexec::constructible_from<_Base, std::in_place_t, __task*, _Args...>
-      __io_task_facade(std::in_place_t, _Args&&... __args) //
+      __io_task_facade(std::in_place_t, _Args&&... __args)
         noexcept(stdexec::__nothrow_constructible_from<_Base, __task*, _Args...>)
         : __task{__vtable}
         , __base_(std::in_place, static_cast<__task*>(this), static_cast<_Args&&>(__args)...) {
@@ -690,7 +701,7 @@ namespace exec {
 
       template <class... _Args>
         requires stdexec::constructible_from<_Base, _Args...>
-      __io_task_facade(std::in_place_t, _Args&&... __args) //
+      __io_task_facade(std::in_place_t, _Args&&... __args)
         noexcept(stdexec::__nothrow_constructible_from<_Base, _Args...>)
         : __task{__vtable}
         , __base_(static_cast<_Args&&>(__args)...) {
@@ -703,7 +714,9 @@ namespace exec {
       void start() & noexcept {
         __context& __context = __base_.context();
         if (__context.submit(this)) {
-          __context.wakeup();
+          if (auto __ec = __context.try_wakeup()) {
+            std::terminate(); // TODO: handle error
+          }
         }
       }
 
@@ -717,7 +730,7 @@ namespace exec {
 
       struct __impl {
         __context& __context_;
-        STDEXEC_ATTRIBUTE((no_unique_address)) _Receiver __receiver_;
+        STDEXEC_ATTRIBUTE(no_unique_address) _Receiver __receiver_;
 
         __impl(__context& __context, _Receiver&& __receiver)
           : __context_{__context}
@@ -817,7 +830,7 @@ namespace exec {
       _Base __base_;
 
       template <class... _Args>
-      __impl_base(__task* __parent, std::in_place_t, _Args&&... __args) //
+      __impl_base(__task* __parent, std::in_place_t, _Args&&... __args)
         noexcept(stdexec::__nothrow_constructible_from<_Base, _Args...>)
         : __parent_{__parent}
         , __base_(static_cast<_Args&&>(__args)...) {
@@ -830,7 +843,7 @@ namespace exec {
       _Base __base_;
 
       template <class... _Args>
-      __impl_base(__task* __parent, std::in_place_t, _Args&&... __args) //
+      __impl_base(__task* __parent, std::in_place_t, _Args&&... __args)
         noexcept(stdexec::__nothrow_constructible_from<_Base, _Args...>)
         : __parent_{__parent}
         , __base_(static_cast<_Args&&>(__args)...) {
@@ -846,8 +859,9 @@ namespace exec {
       using _Receiver = __receiver_of_t<_Base>;
 
       template <class _Ty>
-      static constexpr bool __has_submit_stop_v =
-        requires(_Ty& __base, ::io_uring_sqe& __sqe) { __base.submit_stop(__sqe); };
+      static constexpr bool __has_submit_stop_v = requires(_Ty& __base, ::io_uring_sqe& __sqe) {
+        __base.submit_stop(__sqe);
+      };
 
       using __base_t = __impl_base<_Base, __has_submit_stop_v<_Base>>;
 
@@ -862,7 +876,8 @@ namespace exec {
 
         using __on_context_stop_t = std::optional<stdexec::inplace_stop_callback<__stop_callback>>;
         using __on_receiver_stop_t = std::optional<typename stdexec::stop_token_of_t<
-          stdexec::env_of_t<_Receiver>&>::template callback_type<__stop_callback>>;
+          stdexec::env_of_t<_Receiver>&
+        >::template callback_type<__stop_callback>>;
 
         stdexec::__t<__stop_operation<__impl>> __stop_operation_;
         std::atomic<int> __n_ops_{0};
@@ -871,7 +886,7 @@ namespace exec {
 
         template <class... _Args>
           requires stdexec::constructible_from<_Base, _Args...>
-        __impl(std::in_place_t, __task* __parent, _Args&&... __args) //
+        __impl(std::in_place_t, __task* __parent, _Args&&... __args)
           noexcept(stdexec::__nothrow_constructible_from<_Base, _Args...>)
           : __base_t(__parent, std::in_place, static_cast<_Args&&>(__args)...)
           , __stop_operation_{this} {
@@ -901,8 +916,8 @@ namespace exec {
           __context& __context_ = this->__base_.context();
           _Receiver& __receiver = this->__base_.receiver();
           __on_context_stop_.emplace(__context_.get_stop_token(), __stop_callback{this});
-          __on_receiver_stop_.emplace(
-            stdexec::get_stop_token(stdexec::get_env(__receiver)), __stop_callback{this});
+          __on_receiver_stop_
+            .emplace(stdexec::get_stop_token(stdexec::get_env(__receiver)), __stop_callback{this});
           this->__base_.submit(__sqe);
         }
 
@@ -1023,11 +1038,14 @@ namespace exec {
         }
 
         void complete(const ::io_uring_cqe& __cqe) noexcept {
+          bool const __success =
 #    ifdef STDEXEC_HAS_IO_URING_ASYNC_CANCELLATION
-          if (__cqe.res == -ETIME || __cqe.res == 0) {
+            __cqe.res == -ETIME || __cqe.res == 0
 #    else
-          if (__cqe.res == sizeof(std::uint64_t)) {
+            __cqe.res == sizeof(std::uint64_t)
 #    endif
+            ;
+          if (__success) {
             stdexec::set_value(static_cast<_Receiver&&>(this->__receiver_));
           } else {
             STDEXEC_ASSERT(__cqe.res < 0);
@@ -1061,17 +1079,16 @@ namespace exec {
      public:
       __context* __context_;
 
-      friend auto operator==(const __scheduler& __lhs, const __scheduler& __rhs) -> bool = default;
+      auto operator==(const __scheduler&) const -> bool = default;
 
-      class __schedule_env {
-       public:
-        __context* __context_;
-       private:
-        friend auto tag_invoke(
-          stdexec::get_completion_scheduler_t<stdexec::set_value_t>,
-          const __schedule_env& __env) noexcept -> __scheduler {
-          return __scheduler{__env.__context_};
+      struct __schedule_env {
+        [[nodiscard]]
+        auto query(stdexec::get_completion_scheduler_t<stdexec::set_value_t>) const noexcept
+          -> __scheduler {
+          return __scheduler{__context_};
         }
+
+        __context* __context_;
       };
 
       class __schedule_sender {
@@ -1089,17 +1106,19 @@ namespace exec {
           : __env_{__env} {
         }
 
+        [[nodiscard]]
         auto get_env() const noexcept -> __schedule_env {
           return __env_;
         }
 
+        [[nodiscard]]
         auto get_completion_signatures(stdexec::__ignore = {}) const noexcept -> __completion_sigs {
           return {};
         }
 
         template <stdexec::receiver_of<__completion_sigs> _Receiver>
-        auto connect(_Receiver __receiver) const & //
-          -> stdexec::__t<__schedule_operation<stdexec::__id<_Receiver>>> {
+        auto connect(_Receiver __receiver)
+          const & -> stdexec::__t<__schedule_operation<stdexec::__id<_Receiver>>> {
           return stdexec::__t<__schedule_operation<stdexec::__id<_Receiver>>>(
             std::in_place, *__env_.__context_, static_cast<_Receiver&&>(__receiver));
         }
@@ -1109,7 +1128,8 @@ namespace exec {
         using __completion_sigs = stdexec::completion_signatures<
           stdexec::set_value_t(),
           stdexec::set_error_t(std::exception_ptr),
-          stdexec::set_stopped_t()>;
+          stdexec::set_stopped_t()
+        >;
 
        public:
         using sender_concept = stdexec::sender_t;
@@ -1119,6 +1139,7 @@ namespace exec {
         __schedule_env __env_;
         std::chrono::nanoseconds __duration_;
 
+        [[nodiscard]]
         auto get_env() const noexcept -> __schedule_env {
           return __env_;
         }
@@ -1130,36 +1151,34 @@ namespace exec {
         }
 
         template <stdexec::receiver_of<__completion_sigs> _Receiver>
-        auto connect(_Receiver __receiver) const & //
-          -> stdexec::__t<__schedule_after_operation<stdexec::__id<_Receiver>>> {
+        auto connect(_Receiver __receiver)
+          const & -> stdexec::__t<__schedule_after_operation<stdexec::__id<_Receiver>>> {
           return stdexec::__t<__schedule_after_operation<stdexec::__id<_Receiver>>>(
             std::in_place, *__env_.__context_, __duration_, static_cast<_Receiver&&>(__receiver));
         }
       };
 
+      [[nodiscard]]
       auto schedule() const -> __schedule_sender {
         return __schedule_sender{__schedule_env{__context_}};
       }
 
-      friend auto tag_invoke(exec::now_t, const __scheduler&) noexcept
-        -> std::chrono::time_point<std::chrono::steady_clock> {
+      [[nodiscard]]
+      auto now() const noexcept -> std::chrono::time_point<std::chrono::steady_clock> {
         return std::chrono::steady_clock::now();
       }
 
-      friend auto tag_invoke(
-        exec::schedule_after_t,
-        const __scheduler& __sched,
-        std::chrono::nanoseconds __duration) -> __schedule_after_sender {
-        return __schedule_after_sender{.__env_ = {__sched.__context_}, .__duration_ = __duration};
+      [[nodiscard]]
+      auto schedule_after(std::chrono::nanoseconds __duration) const -> __schedule_after_sender {
+        return __schedule_after_sender{.__env_ = {__context_}, .__duration_ = __duration};
       }
 
       template <class _Clock, class _Duration>
-      friend auto tag_invoke(
-        exec::schedule_at_t,
-        const __scheduler& __sched,
-        const std::chrono::time_point<_Clock, _Duration>& __time_point) -> __schedule_after_sender {
+      [[nodiscard]]
+      auto schedule_at(const std::chrono::time_point<_Clock, _Duration>& __time_point) const
+        -> __schedule_after_sender {
         auto __duration = __time_point - _Clock::now();
-        return __schedule_after_sender{.__env_ = {__sched.__context_}, .__duration_ = __duration};
+        return __schedule_after_sender{.__env_ = {__context_}, .__duration_ = __duration};
       }
     };
 
@@ -1171,6 +1190,9 @@ namespace exec {
   using __io_uring::until;
   using io_uring_context = __io_uring::__context;
   using io_uring_scheduler = __io_uring::__scheduler;
+
+  static_assert(__timed_scheduler<io_uring_scheduler>);
+
 } // namespace exec
 
 #  endif // if __has_include(<linux/verison.h>)
