@@ -32,6 +32,7 @@
 
 #include <algorithm>
 #include <atomic>
+#include <compare>
 #include <condition_variable>
 #include <cstdint>
 #include <exception>
@@ -174,14 +175,14 @@ namespace exec {
         // TODO: code to reconstitute a static_thread_pool_ schedule sender
       };
 
-      template <class SenderId, std::integral Shape, class Fun>
+      template <class SenderId, bool parallelize, std::integral Shape, class Fun>
       struct bulk_sender {
         using Sender = stdexec::__t<SenderId>;
         struct __t;
       };
 
-      template <sender Sender, std::integral Shape, class Fun>
-      using bulk_sender_t = __t<bulk_sender<__id<__decay_t<Sender>>, Shape, Fun>>;
+      template <sender Sender, bool parallelize, std::integral Shape, class Fun>
+      using bulk_sender_t = __t<bulk_sender<__id<__decay_t<Sender>>, parallelize, Shape, Fun>>;
 
 #if STDEXEC_MSVC()
       // MSVCBUG https://developercommunity.visualstudio.com/t/Alias-template-with-pack-expansion-in-no/10437850
@@ -194,50 +195,81 @@ namespace exec {
 #endif
 
       template <class Fun, class Shape, class... Args>
-        requires __callable<Fun, Shape, Args&...>
-      using bulk_non_throwing = //
-        __mbool<
-          // If function invocation doesn't throw
-          __nothrow_callable<Fun, Shape, Args&...> &&
+        requires __callable<Fun, Shape, Shape, Args&...>
+      using bulk_non_throwing = __mbool<
+        // If function invocation doesn't throw
+        __nothrow_callable<Fun, Shape, Shape, Args&...> &&
       // and emplacing a tuple doesn't throw
 #if STDEXEC_MSVC()
-          __bulk_non_throwing<Args...>::__v
+        __bulk_non_throwing<Args...>::__v
 #else
-          noexcept(__decayed_std_tuple<Args...>(std::declval<Args>()...))
+        noexcept(__decayed_std_tuple<Args...>(std::declval<Args>()...))
 #endif
-          // there's no need to advertise completion with `exception_ptr`
-          >;
+        // there's no need to advertise completion with `exception_ptr`
+      >;
 
-      template <class CvrefSender, class Receiver, class Shape, class Fun, bool MayThrow>
+      template <
+        class CvrefSender,
+        class Receiver,
+        bool parallelize,
+        class Shape,
+        class Fun,
+        bool MayThrow
+      >
       struct bulk_shared_state;
 
-      template <class CvrefSenderId, class ReceiverId, class Shape, class Fun, bool MayThrow>
+      template <
+        class CvrefSenderId,
+        class ReceiverId,
+        bool parallelize,
+        class Shape,
+        class Fun,
+        bool MayThrow
+      >
       struct bulk_receiver {
         using CvrefSender = __cvref_t<CvrefSenderId>;
         using Receiver = stdexec::__t<ReceiverId>;
         struct __t;
       };
 
-      template <class CvrefSender, class Receiver, class Shape, class Fun, bool MayThrow>
-      using bulk_receiver_t =
-        __t<bulk_receiver<__cvref_id<CvrefSender>, __id<Receiver>, Shape, Fun, MayThrow>>;
+      template <
+        class CvrefSender,
+        class Receiver,
+        bool parallelize,
+        class Shape,
+        class Fun,
+        bool MayThrow
+      >
+      using bulk_receiver_t = __t<
+        bulk_receiver<__cvref_id<CvrefSender>, __id<Receiver>, parallelize, Shape, Fun, MayThrow>
+      >;
 
-      template <class CvrefSenderId, class ReceiverId, std::integral Shape, class Fun>
+      template <
+        class CvrefSenderId,
+        class ReceiverId,
+        bool parallelize,
+        std::integral Shape,
+        class Fun
+      >
       struct bulk_op_state {
         using CvrefSender = stdexec::__cvref_t<CvrefSenderId>;
         using Receiver = stdexec::__t<ReceiverId>;
         struct __t;
       };
 
-      template <class Sender, class Receiver, std::integral Shape, class Fun>
-      using bulk_op_state_t =
-        __t<bulk_op_state<__id<__decay_t<Sender>>, __id<__decay_t<Receiver>>, Shape, Fun>>;
+      template <class Sender, class Receiver, bool parallelize, std::integral Shape, class Fun>
+      using bulk_op_state_t = __t<
+        bulk_op_state<__id<__decay_t<Sender>>, __id<__decay_t<Receiver>>, parallelize, Shape, Fun>
+      >;
 
       struct transform_bulk {
         template <class Data, class Sender>
-        auto operator()(bulk_t, Data&& data, Sender&& sndr) {
-          auto [shape, fun] = static_cast<Data&&>(data);
-          return bulk_sender_t<Sender, decltype(shape), decltype(fun)>{
+        auto operator()(bulk_chunked_t, Data&& data, Sender&& sndr) {
+          auto [pol, shape, fun] = static_cast<Data&&>(data);
+          using policy_t = std::remove_cvref_t<decltype(pol.__get())>;
+          constexpr bool parallelize = std::same_as<policy_t, parallel_policy>
+                                    || std::same_as<policy_t, parallel_unsequenced_policy>;
+          return bulk_sender_t<Sender, parallelize, decltype(shape), decltype(fun)>{
             pool_, static_cast<Sender&&>(sndr), shape, std::move(fun)};
         }
 
@@ -255,7 +287,7 @@ namespace exec {
       };
 #endif
 
-      static unsigned int _hardware_concurrency() noexcept {
+      static auto _hardware_concurrency() noexcept -> unsigned int {
         unsigned int n = std::thread::hardware_concurrency();
         return n == 0 ? 1 : n;
       }
@@ -263,7 +295,7 @@ namespace exec {
      public:
       struct domain : stdexec::default_domain {
         // For eager customization
-        template <sender_expr_for<bulk_t> Sender>
+        template <sender_expr_for<bulk_chunked_t> Sender>
         auto transform_sender(Sender&& sndr) const noexcept {
           if constexpr (__completes_on<Sender, static_thread_pool_::scheduler>) {
             auto sched = get_completion_scheduler<set_value_t>(get_env(sndr));
@@ -271,26 +303,22 @@ namespace exec {
           } else {
             static_assert(
               __completes_on<Sender, static_thread_pool_::scheduler>,
-              "No static_thread_pool_ instance can be found in the sender's environment "
+              "No static_thread_pool instance can be found in the sender's attributes "
               "on which to schedule bulk work.");
             return not_a_sender<__name_of<Sender>>();
           }
         }
 
-        // transform the generic bulk sender into a parallel thread-pool bulk sender
-        template <sender_expr_for<bulk_t> Sender, class Env>
+        // transform the generic bulk_chunked sender into a parallel thread-pool bulk sender
+        template <sender_expr_for<bulk_chunked_t> Sender, class Env>
         auto transform_sender(Sender&& sndr, const Env& env) const noexcept {
-          if constexpr (__completes_on<Sender, static_thread_pool_::scheduler>) {
-            auto sched = get_completion_scheduler<set_value_t>(get_env(sndr));
-            return __sexpr_apply(static_cast<Sender&&>(sndr), transform_bulk{*sched.pool_});
-          } else if constexpr (__starts_on<Sender, static_thread_pool_::scheduler, Env>) {
+          if constexpr (__starts_on<Sender, static_thread_pool_::scheduler, Env>) {
             auto sched = stdexec::get_scheduler(env);
             return __sexpr_apply(static_cast<Sender&&>(sndr), transform_bulk{*sched.pool_});
           } else {
-            static_assert( //
-              __starts_on<Sender, static_thread_pool_::scheduler, Env>
-                || __completes_on<Sender, static_thread_pool_::scheduler>,
-              "No static_thread_pool_ instance can be found in the sender's or receiver's "
+            static_assert(
+              __starts_on<Sender, static_thread_pool_::scheduler, Env>,
+              "No static_thread_pool instance can be found in the receiver's "
               "environment on which to schedule bulk work.");
             return not_a_sender<__name_of<Sender>>();
           }
@@ -347,8 +375,9 @@ namespace exec {
           using completion_signatures =
             stdexec::completion_signatures<set_value_t(), set_stopped_t()>;
 
+          [[nodiscard]]
           auto get_env() const noexcept -> env {
-            return env{pool_, queue_};
+            return env{.pool_ = pool_, .queue_ = queue_};
           }
 
           template <receiver Receiver>
@@ -420,10 +449,12 @@ namespace exec {
           return _sender{*pool_, queue_, thread_idx_, *nodemask_};
         }
 
+        [[nodiscard]]
         auto query(get_forward_progress_guarantee_t) const noexcept -> forward_progress_guarantee {
           return forward_progress_guarantee::parallel;
         }
 
+        [[nodiscard]]
         auto query(get_domain_t) const noexcept -> domain {
           return {};
         }
@@ -604,6 +635,8 @@ namespace exec {
         void notify_one_sleeping();
         void set_stealing();
         void clear_stealing();
+        void set_sleeping();
+        void clear_sleeping();
 
         bwos::lifo_queue<task_base*, numa_allocator<task_base*>> local_queue_;
         __intrusive_queue<&task_base::next> pending_queue_{};
@@ -620,7 +653,7 @@ namespace exec {
       void run(std::uint32_t index) noexcept;
       void join() noexcept;
 
-      alignas(64) std::atomic<std::uint32_t> numThiefs_{};
+      alignas(64) std::atomic<std::uint32_t> numActive_{};
       alignas(64) remote_queue_list remotes_;
       std::uint32_t threadCount_;
       std::uint32_t maxSteals_{threadCount_ + 1};
@@ -633,10 +666,16 @@ namespace exec {
         int numa_node;
         std::size_t thread_index;
 
-        friend auto operator<(
+        friend auto operator==(
           const thread_index_by_numa_node& lhs,
           const thread_index_by_numa_node& rhs) noexcept -> bool {
-          return lhs.numa_node < rhs.numa_node;
+          return lhs.numa_node == rhs.numa_node;
+        }
+
+        friend auto operator<=>(
+          const thread_index_by_numa_node& lhs,
+          const thread_index_by_numa_node& rhs) noexcept -> std::strong_ordering {
+          return lhs.numa_node <=> rhs.numa_node;
         }
       };
 
@@ -669,9 +708,11 @@ namespace exec {
       for (std::uint32_t index = 0; index < threadCount; ++index) {
         threadStates_[index].emplace(this, index, params, numa_);
         threadIndexByNumaNode_.push_back(
-          thread_index_by_numa_node{threadStates_[index]->numa_node(), index});
+          thread_index_by_numa_node{
+            .numa_node = threadStates_[index]->numa_node(), .thread_index = index});
       }
 
+      // NOLINTNEXTLINE(modernize-use-ranges) we still support platforms without the std::ranges algorithms
       std::sort(threadIndexByNumaNode_.begin(), threadIndexByNumaNode_.end());
       std::vector<workstealing_victim> victims{};
       for (auto& state: threadStates_) {
@@ -682,14 +723,16 @@ namespace exec {
       }
       threads_.reserve(threadCount);
 
-      try {
+      STDEXEC_TRY {
+        numActive_.store(threadCount << 16u, std::memory_order_relaxed);
         for (std::uint32_t i = 0; i < threadCount; ++i) {
           threads_.emplace_back([this, i] { run(i); });
         }
-      } catch (...) {
+      }
+      STDEXEC_CATCH_ALL {
         request_stop();
         join();
-        throw;
+        STDEXEC_THROW();
       }
     }
 
@@ -705,8 +748,9 @@ namespace exec {
     }
 
     inline void static_thread_pool_::run(std::uint32_t threadIndex) noexcept {
-      numa_.bind_to_node(threadStates_[threadIndex]->numa_node());
       STDEXEC_ASSERT(threadIndex < threadCount_);
+      // NOLINTNEXTLINE(bugprone-unused-return-value)
+      numa_.bind_to_node(threadStates_[threadIndex]->numa_node());
       while (true) {
         // Make a blocking call to de-queue a task if we don't already have one.
         auto [task, queueIndex] = threadStates_[threadIndex]->pop();
@@ -730,7 +774,8 @@ namespace exec {
     }
 
     inline auto static_thread_pool_::num_threads(int numa) const noexcept -> std::size_t {
-      thread_index_by_numa_node key{numa, 0};
+      thread_index_by_numa_node key{.numa_node = numa, .thread_index = 0};
+      // NOLINTNEXTLINE(modernize-use-ranges) we still support platforms without the std::ranges algorithms
       auto it = std::lower_bound(threadIndexByNumaNode_.begin(), threadIndexByNumaNode_.end(), key);
       if (it == threadIndexByNumaNode_.end()) {
         return 0;
@@ -756,7 +801,8 @@ namespace exec {
     inline auto
       static_thread_pool_::get_thread_index(int nodeIndex, std::size_t threadIndex) const noexcept
       -> std::size_t {
-      thread_index_by_numa_node key{nodeIndex, 0};
+      thread_index_by_numa_node key{.numa_node = nodeIndex, .thread_index = 0};
+      // NOLINTNEXTLINE(modernize-use-ranges) we still support platforms without the std::ranges algorithms
       auto it = std::lower_bound(threadIndexByNumaNode_.begin(), threadIndexByNumaNode_.end(), key);
       STDEXEC_ASSERT(it != threadIndexByNumaNode_.end());
       std::advance(it, threadIndex);
@@ -869,7 +915,7 @@ namespace exec {
 
     inline auto static_thread_pool_::thread_state::try_remote()
       -> static_thread_pool_::thread_state::pop_result {
-      pop_result result{nullptr, index_};
+      pop_result result{.task = nullptr, .queueIndex = index_};
       __intrusive_queue<&task_base::next> remotes = pool_->remotes_.pop_all_reversed(index_);
       pending_queue_.append(std::move(remotes));
       if (!pending_queue_.empty()) {
@@ -882,7 +928,7 @@ namespace exec {
 
     inline auto static_thread_pool_::thread_state::try_pop()
       -> static_thread_pool_::thread_state::pop_result {
-      pop_result result{nullptr, index_};
+      pop_result result{.task = nullptr, .queueIndex = index_};
       result.task = local_queue_.pop_back();
       if (result.task) [[likely]] {
         return result;
@@ -893,13 +939,13 @@ namespace exec {
     inline auto static_thread_pool_::thread_state::try_steal(std::span<workstealing_victim> victims)
       -> static_thread_pool_::thread_state::pop_result {
       if (victims.empty()) {
-        return {nullptr, index_};
+        return {.task = nullptr, .queueIndex = index_};
       }
       std::uniform_int_distribution<std::uint32_t> dist(
         0, static_cast<std::uint32_t>(victims.size() - 1));
       std::uint32_t victimIndex = dist(rng_);
       auto& v = victims[victimIndex];
-      return {v.try_steal(), v.index()};
+      return {.task = v.try_steal(), .queueIndex = v.index()};
     }
 
     inline auto static_thread_pool_::thread_state::try_steal_near()
@@ -923,12 +969,31 @@ namespace exec {
       pending_queue_.prepend(std::move(tasks));
     }
 
-    inline void static_thread_pool_::thread_state::set_stealing() {
-      pool_->numThiefs_.fetch_add(1, std::memory_order_relaxed);
+    inline void static_thread_pool_::thread_state::set_sleeping() {
+      pool_->numActive_.fetch_sub(1u << 16u, std::memory_order_relaxed);
     }
 
+    // wakeup a worker thread and maintain the invariant that we always one active thief as long as a potential victim is awake
+    inline void static_thread_pool_::thread_state::clear_sleeping() {
+      const std::uint32_t numActive = pool_->numActive_
+                                        .fetch_add(1u << 16u, std::memory_order_relaxed);
+      if (numActive == 0) {
+        notify_one_sleeping();
+      }
+    }
+
+    inline void static_thread_pool_::thread_state::set_stealing() {
+      const std::uint32_t diff = 1u - (1u << 16u);
+      pool_->numActive_.fetch_add(diff, std::memory_order_relaxed);
+    }
+
+    // put a thief to sleep but maintain the invariant that we always have one active thief as long as a potential victim is awake
     inline void static_thread_pool_::thread_state::clear_stealing() {
-      if (pool_->numThiefs_.fetch_sub(1, std::memory_order_relaxed) == 1) {
+      constexpr std::uint32_t diff = 1 - (1u << 16u);
+      const std::uint32_t numActive = pool_->numActive_.fetch_sub(diff, std::memory_order_relaxed);
+      const std::uint32_t numVictims = numActive >> 16u;
+      const std::uint32_t numThiefs = numActive & 0xffffu;
+      if (numThiefs == 1 && numVictims != 0) {
         notify_one_sleeping();
       }
     }
@@ -980,9 +1045,14 @@ namespace exec {
           if (result.task) {
             return result;
           }
+          set_sleeping();
           cv_.wait(lock);
+          lock.unlock();
+          clear_sleeping();
+        }
+        if (lock.owns_lock()) {
+          lock.unlock();
         }
-        lock.unlock();
         state_.store(state::running, std::memory_order_relaxed);
         result = try_pop();
       }
@@ -1061,8 +1131,8 @@ namespace exec {
 
     //////////////////////////////////////////////////////////////////////////////////////////////////
     // What follows is the implementation for parallel bulk execution on static_thread_pool_.
-    template <class SenderId, std::integral Shape, class Fun>
-    struct static_thread_pool_::bulk_sender<SenderId, Shape, Fun>::__t {
+    template <class SenderId, bool parallelize, std::integral Shape, class Fun>
+    struct static_thread_pool_::bulk_sender<SenderId, parallelize, Shape, Fun>::__t {
       using __id = bulk_sender;
       using sender_concept = sender_t;
 
@@ -1072,39 +1142,41 @@ namespace exec {
       Fun fun_;
 
       template <class Sender, class... Env>
-      using with_error_invoke_t = //
-        __if_c<
-          __v<__value_types_t<
-            __completion_signatures_of_t<Sender, Env...>,
-            __mbind_front_q<bulk_non_throwing, Fun, Shape>,
-            __q<__mand>>>,
-          completion_signatures<>,
-          __eptr_completion>;
+      using with_error_invoke_t = __if_c<
+        __v<__value_types_t<
+          __completion_signatures_of_t<Sender, Env...>,
+          __mbind_front_q<bulk_non_throwing, Fun, Shape>,
+          __q<__mand>
+        >>,
+        completion_signatures<>,
+        __eptr_completion
+      >;
 
       template <class... Tys>
       using set_value_t = completion_signatures<set_value_t(stdexec::__decay_t<Tys>...)>;
 
       template <class Self, class... Env>
-      using __completions_t = //
-        stdexec::transform_completion_signatures<
-          __completion_signatures_of_t<__copy_cvref_t<Self, Sender>, Env...>,
-          with_error_invoke_t<__copy_cvref_t<Self, Sender>, Env...>,
-          set_value_t>;
+      using __completions_t = stdexec::transform_completion_signatures<
+        __completion_signatures_of_t<__copy_cvref_t<Self, Sender>, Env...>,
+        with_error_invoke_t<__copy_cvref_t<Self, Sender>, Env...>,
+        set_value_t
+      >;
 
       template <class Self, class Receiver>
-      using bulk_op_state_t = //
-        stdexec::__t<bulk_op_state<__cvref_id<Self, Sender>, stdexec::__id<Receiver>, Shape, Fun>>;
+      using bulk_op_state_t = stdexec::__t<
+        bulk_op_state<__cvref_id<Self, Sender>, stdexec::__id<Receiver>, parallelize, Shape, Fun>
+      >;
 
       template <__decays_to<__t> Self, receiver Receiver>
         requires receiver_of<Receiver, __completions_t<Self, env_of_t<Receiver>>>
-      static auto connect(Self&& self, Receiver rcvr) //
-        noexcept(__nothrow_constructible_from<
-                 bulk_op_state_t<Self, Receiver>,
-                 static_thread_pool_&,
-                 Shape,
-                 Fun,
-                 Sender,
-                 Receiver>) -> bulk_op_state_t<Self, Receiver> {
+      static auto connect(Self&& self, Receiver rcvr) noexcept(__nothrow_constructible_from<
+                                                               bulk_op_state_t<Self, Receiver>,
+                                                               static_thread_pool_&,
+                                                               Shape,
+                                                               Fun,
+                                                               Sender,
+                                                               Receiver
+      >) -> bulk_op_state_t<Self, Receiver> {
         return bulk_op_state_t<Self, Receiver>{
           self.pool_,
           self.shape_,
@@ -1124,7 +1196,14 @@ namespace exec {
     };
 
     //! The customized operation state for `stdexec::bulk` operations
-    template <class CvrefSender, class Receiver, class Shape, class Fun, bool MayThrow>
+    template <
+      class CvrefSender,
+      class Receiver,
+      bool parallelize,
+      class Shape,
+      class Fun,
+      bool MayThrow
+    >
     struct static_thread_pool_::bulk_shared_state {
       //! The actual `bulk_task` holds a pointer to the shared state
       //! and its `__execute` function reads from that shared state.
@@ -1142,9 +1221,7 @@ namespace exec {
               // In the case that the shape is much larger than the total number of threads,
               // then each call to computation will call the function many times.
               auto [begin, end] = even_share(sh_state.shape_, tid, total_threads);
-              for (Shape i = begin; i < end; ++i) {
-                sh_state.fun_(i, args...);
-              }
+              sh_state.fun_(begin, end, args...);
             };
 
             auto completion = [&](auto&... args) {
@@ -1152,9 +1229,10 @@ namespace exec {
             };
 
             if constexpr (MayThrow) {
-              try {
+              STDEXEC_TRY {
                 sh_state.apply(computation);
-              } catch (...) {
+              }
+              STDEXEC_CATCH_ALL {
                 std::uint32_t expected = total_threads;
 
                 if (sh_state.thread_with_exception_.compare_exchange_strong(
@@ -1188,12 +1266,12 @@ namespace exec {
         }
       };
 
-      using variant_t = //
-        __value_types_of_t<
-          CvrefSender,
-          env_of_t<Receiver>,
-          __q<__decayed_std_tuple>,
-          __q<__nullable_std_variant>>;
+      using variant_t = __value_types_of_t<
+        CvrefSender,
+        env_of_t<Receiver>,
+        __q<__decayed_std_tuple>,
+        __q<__nullable_std_variant>
+      >;
 
       variant_t data_;
       static_thread_pool_& pool_;
@@ -1210,8 +1288,12 @@ namespace exec {
       //! That is, we don't need an agent for each of the shape values.
       [[nodiscard]]
       auto num_agents_required() const -> std::uint32_t {
-        return static_cast<std::uint32_t>(
-          std::min(shape_, static_cast<Shape>(pool_.available_parallelism())));
+        if constexpr (parallelize) {
+          return static_cast<std::uint32_t>(
+            std::min(shape_, static_cast<Shape>(pool_.available_parallelism())));
+        } else {
+          return static_cast<std::uint32_t>(1);
+        }
       }
 
       template <class F>
@@ -1240,18 +1322,33 @@ namespace exec {
     };
 
     //! A customized receiver to allow parallel execution of `stdexec::bulk` operations:
-    template <class CvrefSenderId, class ReceiverId, class Shape, class Fun, bool MayThrow>
-    struct static_thread_pool_::bulk_receiver<CvrefSenderId, ReceiverId, Shape, Fun, MayThrow>::__t {
+    template <
+      class CvrefSenderId,
+      class ReceiverId,
+      bool parallelize,
+      class Shape,
+      class Fun,
+      bool MayThrow
+    >
+    struct static_thread_pool_::bulk_receiver<
+      CvrefSenderId,
+      ReceiverId,
+      parallelize,
+      Shape,
+      Fun,
+      MayThrow
+    >::__t {
       using __id = bulk_receiver;
       using receiver_concept = receiver_t;
 
-      using shared_state = bulk_shared_state<CvrefSender, Receiver, Shape, Fun, MayThrow>;
+      using shared_state =
+        bulk_shared_state<CvrefSender, Receiver, parallelize, Shape, Fun, MayThrow>;
 
       shared_state& shared_state_;
 
       void enqueue() noexcept {
-        shared_state_.pool_.bulk_enqueue(
-          shared_state_.tasks_.data(), shared_state_.num_agents_required());
+        shared_state_.pool_
+          .bulk_enqueue(shared_state_.tasks_.data(), shared_state_.num_agents_required());
       }
 
       template <class... As>
@@ -1261,9 +1358,10 @@ namespace exec {
         shared_state& state = shared_state_;
 
         if constexpr (MayThrow) {
-          try {
+          STDEXEC_TRY {
             state.data_.template emplace<tuple_t>(static_cast<As&&>(as)...);
-          } catch (...) {
+          }
+          STDEXEC_CATCH_ALL {
             stdexec::set_error(std::move(state.rcvr_), std::current_exception());
           }
         } else {
@@ -1273,9 +1371,8 @@ namespace exec {
         if (state.shape_) {
           enqueue();
         } else {
-          state.apply([&](auto&... args) {
-            stdexec::set_value(std::move(state.rcvr_), std::move(args)...);
-          });
+          state.apply(
+            [&](auto&... args) { stdexec::set_value(std::move(state.rcvr_), std::move(args)...); });
         }
       }
 
@@ -1295,19 +1392,26 @@ namespace exec {
       }
     };
 
-    template <class CvrefSenderId, class ReceiverId, std::integral Shape, class Fun>
-    struct static_thread_pool_::bulk_op_state<CvrefSenderId, ReceiverId, Shape, Fun>::__t {
+    template <class CvrefSenderId, class ReceiverId, bool parallelize, std::integral Shape, class Fun>
+    struct static_thread_pool_::bulk_op_state<
+      CvrefSenderId,
+      ReceiverId,
+      parallelize,
+      Shape,
+      Fun
+    >::__t {
       using __id = bulk_op_state;
 
-      static constexpr bool may_throw = //
-        !__v<__value_types_of_t<
-          CvrefSender,
-          env_of_t<Receiver>,
-          __mbind_front_q<bulk_non_throwing, Fun, Shape>,
-          __q<__mand>>>;
+      static constexpr bool may_throw = !__v<__value_types_of_t<
+        CvrefSender,
+        env_of_t<Receiver>,
+        __mbind_front_q<bulk_non_throwing, Fun, Shape>,
+        __q<__mand>
+      >>;
 
-      using bulk_rcvr = bulk_receiver_t<CvrefSender, Receiver, Shape, Fun, may_throw>;
-      using shared_state = bulk_shared_state<CvrefSender, Receiver, Shape, Fun, may_throw>;
+      using bulk_rcvr = bulk_receiver_t<CvrefSender, Receiver, parallelize, Shape, Fun, may_throw>;
+      using shared_state =
+        bulk_shared_state<CvrefSender, Receiver, parallelize, Shape, Fun, may_throw>;
       using inner_op_state = connect_result_t<CvrefSender, bulk_rcvr>;
 
       shared_state shared_state_;
@@ -1348,7 +1452,7 @@ namespace exec {
         __intrusive_queue<&task_base::next> tasks_{};
         std::size_t tasks_size_{};
         std::atomic<std::size_t> countdown_{std::ranges::size(range_)};
-      }; // namespace schedule_all_
+      };
 
       template <class Range, class ItemReceiverId>
       struct item_operation {
@@ -1474,7 +1578,8 @@ namespace exec {
           using ItemOperation = connect_result_t<NextSender, NextReceiver>;
 
           using ItemAllocator = typename std::allocator_traits<Allocator>::template rebind_alloc<
-            stdexec::__manual_lifetime<ItemOperation>>;
+            stdexec::__manual_lifetime<ItemOperation>
+          >;
 
           std::vector<__manual_lifetime<ItemOperation>, ItemAllocator> items_;
 
@@ -1484,7 +1589,8 @@ namespace exec {
           __t(Range range, static_thread_pool_& pool, Receiver rcvr)
             : operation_base_with_receiver<
                 Range,
-                Receiver>{std::move(range), pool, static_cast<Receiver&&>(rcvr)}
+                Receiver
+              >{std::move(range), pool, static_cast<Receiver&&>(rcvr)}
             , items_(std::ranges::size(this->range_), ItemAllocator(get_allocator(this->rcvr_))) {
           }
 
@@ -1535,8 +1641,6 @@ namespace exec {
 
       template <class Range>
       class sequence<Range>::__t {
-        using item_sender_t = stdexec::__t<item_sender<Range>>;
-
         Range range_;
         static_thread_pool_* pool_;
 
@@ -1545,8 +1649,11 @@ namespace exec {
 
         using sender_concept = sequence_sender_t;
 
-        using completion_signatures = stdexec::
-          completion_signatures<set_value_t(), set_error_t(std::exception_ptr), set_stopped_t()>;
+        using completion_signatures = stdexec::completion_signatures<
+          set_value_t(),
+          set_error_t(std::exception_ptr),
+          set_stopped_t()
+        >;
 
         using item_types = exec::item_types<stdexec::__t<item_sender<Range>>>;
 
@@ -1555,11 +1662,17 @@ namespace exec {
           , pool_(&pool) {
         }
 
-       private:
-        template <__decays_to<__t> Self, exec::sequence_receiver_of<item_types> Receiver>
-        friend auto tag_invoke(exec::subscribe_t, Self&& self, Receiver rcvr) noexcept
+        template <exec::sequence_receiver_of<item_types> Receiver>
+        auto subscribe(Receiver rcvr) && noexcept
           -> stdexec::__t<operation<Range, stdexec::__id<Receiver>>> {
-          return {static_cast<Range&&>(self.range_), *self.pool_, static_cast<Receiver&&>(rcvr)};
+          return {static_cast<Range&&>(range_), *pool_, static_cast<Receiver&&>(rcvr)};
+        }
+
+        template <exec::sequence_receiver_of<item_types> Receiver>
+          requires __decay_copyable<Range const &>
+        auto subscribe(Receiver rcvr) const & noexcept
+          -> stdexec::__t<operation<Range, stdexec::__id<Receiver>>> {
+          return {range_, *pool_, static_cast<Receiver&&>(rcvr)};
         }
       };
     } // namespace schedule_all_
