@@ -27,7 +27,6 @@
 
 #include "any_sender_of.hpp"
 #include "at_coroutine_exit.hpp"
-#include "inline_scheduler.hpp"
 #include "scope.hpp"
 
 STDEXEC_PRAGMA_PUSH()
@@ -48,25 +47,20 @@ namespace exec {
     static_assert(scheduler<__any_scheduler>);
 
     template <class _Ty>
-    concept __stop_token_provider = //
-      requires(const _Ty& t) {      //
-        get_stop_token(t);
-      };
+    concept __stop_token_provider = requires(const _Ty& t) { get_stop_token(t); };
 
     template <class _Ty>
-    concept __indirect_stop_token_provider = //
-      requires(const _Ty& t) {
-        { get_env(t) } -> __stop_token_provider;
-      };
+    concept __indirect_stop_token_provider = requires(const _Ty& t) {
+      { get_env(t) } -> __stop_token_provider;
+    };
 
     template <class _Ty>
-    concept __indirect_scheduler_provider = //
-      requires(const _Ty& t) {
-        { get_env(t) } -> __scheduler_provider;
-      };
+    concept __indirect_scheduler_provider = requires(const _Ty& t) {
+      { get_env(t) } -> __scheduler_provider;
+    };
 
     template <class _ParentPromise>
-    constexpr bool __check_parent_promise_has_scheduler() noexcept {
+    constexpr auto __check_parent_promise_has_scheduler() noexcept -> bool {
       static_assert(
         __indirect_scheduler_provider<_ParentPromise>,
         "exec::task<T> cannot be co_await-ed in a coroutine that "
@@ -93,8 +87,6 @@ namespace exec {
       __sticky
     };
 
-    struct __parent_promise_t { };
-
     template <__scheduler_affinity _SchedulerAffinity = __scheduler_affinity::__sticky>
     class __default_task_context_impl {
       template <class _ParentPromise>
@@ -102,13 +94,13 @@ namespace exec {
 
       static constexpr bool __with_scheduler = _SchedulerAffinity == __scheduler_affinity::__sticky;
 
-      STDEXEC_ATTRIBUTE((no_unique_address)) __if_c<__with_scheduler, __any_scheduler, __ignore> //
-        __scheduler_{exec::inline_scheduler{}};
+      STDEXEC_ATTRIBUTE(no_unique_address)
+      __if_c<__with_scheduler, __any_scheduler, __ignore> __scheduler_{stdexec::inline_scheduler{}};
       inplace_stop_token __stop_token_;
 
      public:
       template <class _ParentPromise>
-      explicit __default_task_context_impl(__parent_promise_t, _ParentPromise& __parent) noexcept {
+      explicit __default_task_context_impl(_ParentPromise& __parent) noexcept {
         if constexpr (_SchedulerAffinity == __scheduler_affinity::__sticky) {
           if constexpr (__check_parent_promise_has_scheduler<_ParentPromise>()) {
             __scheduler_ = get_scheduler(get_env(__parent));
@@ -121,12 +113,14 @@ namespace exec {
         : __scheduler_{static_cast<_Scheduler&&>(__scheduler)} {
       }
 
+      [[nodiscard]]
       auto query(get_scheduler_t) const noexcept -> const __any_scheduler&
         requires(__with_scheduler)
       {
         return __scheduler_;
       }
 
+      [[nodiscard]]
       auto query(get_stop_token_t) const noexcept -> inplace_stop_token {
         return __stop_token_;
       }
@@ -163,8 +157,8 @@ namespace exec {
     struct __default_awaiter_context {
       template <__scheduler_affinity _Affinity>
       explicit __default_awaiter_context(
-        __default_task_context_impl<_Affinity>& __self,
-        _ParentPromise& __parent) noexcept {
+        __default_task_context_impl<_Affinity>&,
+        _ParentPromise&) noexcept {
       }
     };
 
@@ -188,8 +182,11 @@ namespace exec {
         : __stop_callback_{
             get_stop_token(get_env(__parent)),
             __forward_stop_request{__stop_source_}} {
-        static_assert(
-          std::is_nothrow_constructible_v<__stop_callback_t, __stop_token_t, __forward_stop_request>);
+        static_assert(std::is_nothrow_constructible_v<
+                      __stop_callback_t,
+                      __stop_token_t,
+                      __forward_stop_request
+        >);
         __self.__stop_token_ = __stop_source_.get_token();
       }
 
@@ -228,8 +225,8 @@ namespace exec {
     struct __default_awaiter_context<void> {
       template <__scheduler_affinity _Affinity, class _ParentPromise>
       explicit __default_awaiter_context(
-        __default_task_context_impl<_Affinity>& __self,
-        _ParentPromise& __parent) noexcept {
+        __default_task_context_impl<_Affinity>&,
+        _ParentPromise&) noexcept {
       }
 
       template <__scheduler_affinity _Affinity, __indirect_stop_token_provider _ParentPromise>
@@ -245,8 +242,8 @@ namespace exec {
         if constexpr (std::same_as<__stop_token_t, inplace_stop_token>) {
           __self.__stop_token_ = get_stop_token(get_env(__parent));
         } else if (auto __token = get_stop_token(get_env(__parent)); __token.stop_possible()) {
-          __stop_callback_.emplace<__stop_callback_t>(
-            std::move(__token), __forward_stop_request{__stop_source_});
+          __stop_callback_
+            .emplace<__stop_callback_t>(std::move(__token), __forward_stop_request{__stop_source_});
           __self.__stop_token_ = __stop_source_.get_token();
         }
       }
@@ -256,9 +253,8 @@ namespace exec {
     };
 
     template <class _Promise, class _ParentPromise = void>
-    using awaiter_context_t =                //
-      typename __decay_t<env_of_t<_Promise>> //
-      ::template awaiter_context_t<_Promise, _ParentPromise>;
+    using awaiter_context_t =
+      typename __decay_t<env_of_t<_Promise>>::template awaiter_context_t<_Promise, _ParentPromise>;
 
     ////////////////////////////////////////////////////////////////////////////////
     // In a base class so it can be specialized when _Ty is void:
@@ -282,6 +278,25 @@ namespace exec {
       __variant_for<__void, std::exception_ptr> __data_{};
     };
 
+    template <class _Sch>
+    struct __just_void {
+      using sender_concept = sender_t;
+      using completion_signatures = stdexec::completion_signatures<set_value_t()>;
+
+      template <class _Rcvr>
+      [[nodiscard]]
+      static constexpr auto connect(_Rcvr __rcvr) noexcept {
+        return stdexec::connect(just(), static_cast<_Rcvr&&>(__rcvr));
+      }
+
+      [[nodiscard]]
+      constexpr auto get_env() const noexcept {
+        return prop{get_completion_scheduler<set_value_t>, __sch_};
+      }
+
+      _Sch __sch_;
+    };
+
     enum class disposition : unsigned {
       stopped,
       succeeded,
@@ -320,6 +335,9 @@ namespace exec {
       }
 
      private:
+      using __scheduler_t =
+        __query_result_or_t<get_scheduler_t, _Context, stdexec::inline_scheduler>;
+
       struct __final_awaitable {
         static constexpr auto await_ready() noexcept -> bool {
           return false;
@@ -370,12 +388,13 @@ namespace exec {
           this->__data_.template emplace<1>(std::current_exception());
         }
 
+#ifndef __clang_analyzer__
         template <sender _Awaitable>
           requires __scheduler_provider<_Context>
         auto await_transform(_Awaitable&& __awaitable) noexcept -> decltype(auto) {
           // TODO: If we have a complete-where-it-starts query then we can optimize
           // this to avoid the reschedule
-          return as_awaitable(
+          return stdexec::as_awaitable(
             continues_on(static_cast<_Awaitable&&>(__awaitable), get_scheduler(*__context_)),
             *this);
         }
@@ -395,7 +414,13 @@ namespace exec {
             (void) __cleanup_task.await_resume();
           }
           __context_->set_scheduler(__box.__sched_);
-          return as_awaitable(schedule(__box.__sched_), *this);
+          return stdexec::as_awaitable(schedule(__box.__sched_), *this);
+        }
+#endif
+
+        template <__sender_adaptor_closure_for<__just_void<__scheduler_t>> _Closure>
+        auto await_transform(_Closure&& __closure) noexcept -> decltype(auto) {
+          return await_transform(static_cast<_Closure&&>(__closure)(__just_void<__scheduler_t>()));
         }
 
         template <class _Awaitable>
@@ -430,7 +455,7 @@ namespace exec {
         auto await_suspend(__coro::coroutine_handle<_ParentPromise2> __parent) noexcept
           -> __coro::coroutine_handle<> {
           static_assert(__one_of<_ParentPromise, _ParentPromise2, void>);
-          __coro_.promise().__context_.emplace(__parent_promise_t(), __parent.promise());
+          __coro_.promise().__context_.emplace(__parent.promise());
           __context_.emplace(*__coro_.promise().__context_, __parent.promise());
           __coro_.promise().set_continuation(__parent);
           if constexpr (requires { __coro_.promise().stop_requested() ? 0 : 1; }) {
@@ -442,9 +467,7 @@ namespace exec {
 
         auto await_resume() -> _Ty {
           __context_.reset();
-          scope_guard __on_exit{[this]() noexcept {
-            std::exchange(__coro_, {}).destroy();
-          }};
+          scope_guard __on_exit{[this]() noexcept { std::exchange(__coro_, {}).destroy(); }};
           if (__coro_.promise().__data_.index() == 1)
             std::rethrow_exception(std::move(__coro_.promise().__data_.template get<1>()));
           if constexpr (!std::is_void_v<_Ty>)
@@ -458,10 +481,10 @@ namespace exec {
         requires constructible_from<
           awaiter_context_t<__promise, _ParentPromise>,
           __promise_context_t&,
-          _ParentPromise&>
-      STDEXEC_MEMFN_DECL(auto as_awaitable)(this basic_task&& __self, _ParentPromise&) noexcept
-        -> __task_awaitable<_ParentPromise> {
-        return __task_awaitable<_ParentPromise>{std::exchange(__self.__coro_, {})};
+          _ParentPromise&
+        >
+      auto as_awaitable(_ParentPromise&) && noexcept -> __task_awaitable<_ParentPromise> {
+        return __task_awaitable<_ParentPromise>{std::exchange(__coro_, {})};
       }
 
       // Make this task generally awaitable:
@@ -471,22 +494,6 @@ namespace exec {
         return __task_awaitable<>{std::exchange(__coro_, {})};
       }
 
-      // From the list of types [_Ty], remove any types that are void, and send
-      //   the resulting list to __qf<set_value_t>, which uses the list of types
-      //   as arguments of a function type. In other words, set_value_t() if _Ty
-      //   is void, and set_value_t(_Ty) otherwise.
-      using __set_value_sig_t = __minvoke<__mremove<void, __qf<set_value_t>>, _Ty>;
-
-      // Specify basic_task's completion signatures
-      //   This is only necessary when basic_task is not generally awaitable
-      //   owing to constraints imposed by its _Context parameter.
-      using __task_traits_t = //
-        completion_signatures<__set_value_sig_t, set_error_t(std::exception_ptr), set_stopped_t()>;
-
-      auto get_completion_signatures(__ignore = {}) const -> __task_traits_t {
-        return {};
-      }
-
       explicit basic_task(__coro::coroutine_handle<promise_type> __coro) noexcept
         : __coro_(__coro) {
       }
