@@ -16,9 +16,16 @@ struct no_sync_policy
     operator _ExecutionPolicy() { return __exec_; }
 };
 
-namespace std::experimental
-{
-template<> struct is_execution_policy<no_sync_policy<std::execution::parallel_policy>> : std::true_type{};
+namespace std {
+namespace experimental {
+inline namespace __p1673_version_0 {
+namespace linalg {
+namespace impl {
+template<> inline constexpr bool is_custom_linalg_execution_policy_v<no_sync_policy<std::execution::parallel_policy>> = true;
+}
+}
+}
+}
 }
 
 template <typename _ExecutionPolicy>
