@@ -320,6 +320,7 @@ using __enable_if_EP = typename std::enable_if<
     #define THRUST_DEVICE_SYSTEM THRUST_DEVICE_SYSTEM_OMP
   #endif
 
+  // Stdpar has its own checks for these. Suppress the CCCL checks.
   #define THRUST_IGNORE_DEPRECATED_CPP_11
   #define THRUST_IGNORE_DEPRECATED_CPP_DIALECT 1
   #define CUB_IGNORE_DEPRECATED_CPP_DIALECT 1
@@ -327,15 +328,19 @@ using __enable_if_EP = typename std::enable_if<
   #define CCCL_IGNORE_DEPRECATED_CUDA_BELOW_12 1
 
   #define __NVCOMPILER_PROCESSING_THRUST_INCLUDES
+  #include <thrust/version.h>
+  #if THRUST_VERSION < 200800
+    #error An unexpected version of Thrust which is incompatible with NVC++ is on the include path. NVC++ includes its own version of Thrust; no user-supplied version should be needed.
+  #endif
+
+  #if _NVHPC_STDPAR_GPU && THRUST_VERSION >= 300000 && CUDA_VERSION < 12000
+    #error CUDA 12 or newer is required for GPU offloading of parallel algorithms (-stdpar=gpu)
+  #endif
+
   #include <thrust/execution_policy.h>
   #if _NVHPC_STDPAR_GPU
     #include <thrust/detail/caching_allocator.h>
   #endif
-
-  #include <thrust/version.h>
-  #if THRUST_VERSION < 200600
-    #error An unexpected version of Thrust which is incompatible with NVC++ is on the include path. NVC++ includes its own version of Thrust; no user-supplied version should be needed.
-  #endif
   #undef __NVCOMPILER_PROCESSING_THRUST_INCLUDES
 #endif
 
@@ -418,4 +423,141 @@ template <> struct __policy_for<__back_end::__thrust_gpu> {
   #define _NVHPC_STDPAR_NVTX_RANGE
 #endif
 
+namespace std { 
+namespace __stdpar { 
+namespace __detail {
+
+// min and max functions where the argument may be of different types and
+// where things are passed and returned by value.  These are designed for
+// when the arguments are arithmetic types.
+
+template <class _T, class _U>
+inline typename std::common_type<_T, _U>::type min(_T __a, _U __b) {
+  return __b < __a ? __b : __a;
+}
+
+template <class _T, class _U>
+inline typename std::common_type<_T, _U>::type max(_T __a, _U __b) {
+  return __b < __a ? __a : __b;
+}
+
+// Integer division, but round up instead of down.
+template <class _T> inline _T __div_round_up(_T __num, _T __denom) {
+  return (__num + (__denom - 1)) / __denom;
+}
+
+#if _NVHPC_STDPAR_GPU || __NVCOMPILER_STDPAR_OPENACC_GPU
+
+inline cudaDeviceProp* __getDeviceProperties_uncached() {
+  static cudaDeviceProp __device_properties;
+  cudaError_t __cuda_result = cudaGetDeviceProperties(&__device_properties, 0);
+  if (__cuda_result != cudaSuccess) {
+    throw thrust::system_error(__cuda_result, thrust::cuda_category(),
+                               "Failed to get GPU properties\n");
+  }
+  return &__device_properties;
+}
+inline cudaDeviceProp* __getDeviceProperties() {
+  static cudaDeviceProp* __device_properties = __getDeviceProperties_uncached();
+  return __device_properties;
+}
+
+inline int __get_device_sm_count() {
+  return __getDeviceProperties()->multiProcessorCount;
+}
+inline int __get_device_total_thread_count() {
+  cudaDeviceProp* __device_properties = __getDeviceProperties();
+  return __device_properties->multiProcessorCount *
+      __device_properties->maxThreadsPerMultiProcessor;
+}
+
+template <class _Index>
+_Index __iterations_for_reduce_or_scan(_Index __input_size) {
+  if (__input_size < 1024) {
+    return 0;
+  }
+  if (__input_size < (1 << 22)) {
+    return __get_device_sm_count() *
+        std::lround(0.0614 * std::sqrt(__input_size));
+  }
+  return (__get_device_sm_count() * 128) *
+      std::lround(0.95 * std::sqrt(__input_size) / (8192) + 0.95);
+}
+
+#endif
+
+template <class _Index>
+inline _Index __chunk_start(_Index __chunk_no, _Index __chunk_size,
+                            _Index __leftover) {
+  return (__chunk_no * (__chunk_size + 1)) -
+         ((__chunk_no - __leftover) * _Index(__chunk_no >= __leftover));
+}
+template <class _Index>
+inline _Index __chunk_end(_Index __chunk_no, _Index __chunk_size,
+                          _Index __leftover) {
+  return ((__chunk_no + 1) * (__chunk_size + 1)) -
+         ((__chunk_no + 1 - __leftover) * _Index(__chunk_no + 1 >= __leftover));
+}
+
+template <class _It1, class _It2> struct __either_or_iterator {
+  static_assert(
+      std::is_same<typename std::iterator_traits<_It1>::value_type,
+                   typename std::iterator_traits<_It2>::value_type>::value,
+      "internal error: iterator value types must match");
+  using value_type = typename std::iterator_traits<_It1>::value_type;
+  using difference_type = typename std::common_type<
+      typename std::iterator_traits<_It1>::difference_type,
+      typename std::iterator_traits<_It2>::difference_type>::type;
+  _It1 __it1;
+  _It2 __it2;
+  bool __use1;
+  explicit __either_or_iterator(_It1 __it) : __it1(__it), __use1(true) { }
+  explicit __either_or_iterator(_It2 __it) : __it2(__it), __use1(false) { }
+  __either_or_iterator(_It1 __it1, _It2 __it2, bool __use1)
+      : __it1(__it1), __it2(__it2), __use1(__use1) { }
+  value_type& operator*() {
+    if (__use1) {
+      return *__it1;
+    } else {
+      return *__it2;
+    }
+  }
+  value_type& operator[](difference_type __n) {
+    if (__use1) {
+      return __it1[__n];
+    } else {
+      return __it2[__n];
+    }
+  }
+  __either_or_iterator operator+(difference_type __n) const {
+    if (__use1) {
+      return __either_or_iterator{ __it1 + __n };
+    } else {
+      return __either_or_iterator{ __it2 + __n };
+    }
+  }
+};
+
+template <class _It> struct __either_or_iterator<_It, _It> {
+  using value_type = typename std::iterator_traits<_It>::value_type;
+  using difference_type = typename std::iterator_traits<_It>::difference_type;
+  _It __it;
+  explicit __either_or_iterator(_It __it) : __it(__it) { }
+  __either_or_iterator(_It __it1, _It __it2, bool __use1)
+      : __it(__use1 ? __it1 : __it2) { }
+  value_type& operator*() { return *__it; }
+  value_type& operator[](difference_type __n) { return __it[__n]; }
+  __either_or_iterator operator+(difference_type __n) const {
+    return __either_or_iterator{__it + __n};
+  }
+};
+
+template <class _It1, class _It2>
+__either_or_iterator<_It1, _It2> __make_either_or(_It1 __it1, _It2 __it2,
+                                                  bool __use1) {
+  return __either_or_iterator<_It1, _It2>{__it1, __it2, __use1};
+}
+
+}}}
+
 #endif
